var express = require('express'),
	usersManager = require('./components/usersManager'),
	placesManager = require('./components/placesManager'),
	checkinManager = require('./components/checkinManager');

var FMFServer = express();

FMFServer.configure(function () {
    FMFServer.use(express.logger('dev'));     /* 'default', 'short', 'tiny', 'dev' */
    FMFServer.use(express.bodyParser());
});

/***************** Users *****************/

// Create user
FMFServer.post('/users/createUser',usersManager.createUser);

// Login User
FMFServer.post('/users/loginUser',usersManager.loginUser);

// Delete user
FMFServer.delete('/users/deleteUser/:username/:email/:password', usersManager.deleteUser);

// Update user
FMFServer.put('/users/updateUser', usersManager.updateUser);

// Find user by id
FMFServer.get('/users/findUserById', usersManager.findUserById);

// Find user by username
FMFServer.get('/users/findUserByUsername/:username', usersManager.findUserByUsername);

/***************** Places *****************/

// Create place
FMFServer.post('/places/createPlace',placesManager.createPlace);

// Create branch
FMFServer.post('/places/createBranch',placesManager.createBranch);

// Rank place
FMFServer.put('/places/rankPlace',placesManager.rankPlace);

//find data of the place
FMFServer.get('/places/getDataByPlaceId/:_id',placesManager.getDataByPlaceId);

//find data of the place
FMFServer.get('/places/findPlace/:field/:content',placesManager.findPlace);

//find data of the branch
FMFServer.get('/places/getDataByBranchId/:_id',placesManager.getDataByBranchId);

//Find branch by location
FMFServer.get('/places/getBranchesByLocation/:longitude/:latitude/:maxDistance', placesManager.getBranchesByLocation);

/***************** TO-DO IN PLACES *****************/

// Update place
FMFServer.put('/places/updatePlace',placesManager.updatePlace);
//curl..

// Update branch
FMFServer.put('/places/updateBranch',placesManager.updateBranch);
//curl..

// Delete place
FMFServer.delete('/places/deletePlace/:placename',placesManager.deletePlace);
//curl...

// Delete branch
FMFServer.delete('/places/deleteBranch/:placename/:branchname',placesManager.updateBranch);
//curl..

//create Dish
FMFServer.post('/places/createDish',placesManager.createDish);
//curl..

//Delete Dish
FMFServer.delete('/places/deleteDish/:placename/:dishname',placesManager.deleteDish);
//curl..

// Rank Dish
FMFServer.put('/places/rankDish',placesManager.rankDish);
//curl..

//Find Dish
FMFServer.get('/places/findDish/:dishname',placesManager.findDish);
//curl..


/****************** Checkin ****************/
// Create checkin
FMFServer.post('/checkin/createCheckin', checkinManager.createCheckIn);

// Find checkin By Id
FMFServer.get('/checkin/findCheckinById/:_id', checkinManager.findCheckInById);

// Find checkin By username
FMFServer.get('/checkin/findCheckin/:username', checkinManager.findCheckIn);

/***************** TO-DO IN CHEKIN *****************/

// Update checkin
FMFServer.put('/checkin/updateCheckin', checkinManager.updateCheckIn);

// Delete checkin
FMFServer.delete('/checkin/deleteCheckin/:username', checkinManager.deleteCheckIn);

/*
/***************** TO-DO *****************/
/*
// Upload pictures
FMFServer.post('somePath',somethingHere());

// Commentaries
FMFServer.post('somePath',somethingHere());
*/

var port = process.env.PORT || 5000;

FMFServer.listen(port, function(){
	console.log("FMF is listening on port: " + port);
});